import xbmc
import xbmcgui
import xbmcaddon
import xbmcvfs
import os
import sys
import urllib.request
import datetime
import json
import time
import xml.etree.ElementTree as ET

# Add pytz module to Python path
ADDON = xbmcaddon.Addon()
ADDON_PATH = ADDON.getAddonInfo('path')
PYTZ_PATH = os.path.join(ADDON_PATH, 'script.module.pytz')
if PYTZ_PATH not in sys.path:
    sys.path.insert(0, PYTZ_PATH)

# Now import pytz
import pytz

# Addon paths
KOD_ADDON_PATH = os.path.join(xbmcvfs.translatePath('special://home/addons'), 'plugin.video.kod')
CHANNELS_JSON_URL = 'https://gitlab.com/personal4143605/kod-domains-update/-/raw/main/channels.json?inline=false'
CHANNELS_JSON_PATH = os.path.join(KOD_ADDON_PATH, 'channels.json')

# GitLab API details
GITLAB_API_URL = 'https://gitlab.com/api/v4/projects/personal4143605%2Fkod-domains-update/repository/files/channels.json?ref=main'

# Create Rome timezone
ROME_TZ = pytz.timezone('Europe/Rome')

def get_file_modification_date():
    try:
        timestamp = os.path.getmtime(CHANNELS_JSON_PATH)
        # Convert to Rome timezone
        local_time = datetime.datetime.fromtimestamp(timestamp, tz=ROME_TZ)
        return local_time.strftime('%d/%m/%Y %H:%M:%S')
    except Exception:
        return 'Data non disponibile'

def get_gitlab_file_last_modified_date():
    try:
        req = urllib.request.Request(
            GITLAB_API_URL,
            headers={
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
                'Referer': 'https://gitlab.com/'
            }
        )
        
        with urllib.request.urlopen(req, timeout=10) as response:
            data = json.loads(response.read().decode('utf-8'))
            
        last_commit_id = data.get('last_commit_id')
        if not last_commit_id:
            raise ValueError("No commit ID found")
        
        commit_url = f'https://gitlab.com/api/v4/projects/personal4143605%2Fkod-domains-update/repository/commits/{last_commit_id}'
        req = urllib.request.Request(
            commit_url,
            headers={
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
                'Referer': 'https://gitlab.com/'
            }
        )
        
        with urllib.request.urlopen(req, timeout=10) as response:
            commit_data = json.loads(response.read().decode('utf-8'))
        
        commit_date = commit_data.get('committed_date')
        if commit_date:
            # Convert UTC to Rome timezone
            utc_time = datetime.datetime.strptime(commit_date, '%Y-%m-%dT%H:%M:%S.%f%z')
            rome_time = utc_time.astimezone(ROME_TZ)
            formatted_date = rome_time.strftime('%d/%m/%Y %H:%M:%S')
            ADDON.setSetting('last_gitlab_date', formatted_date)
            return formatted_date
        
        raise ValueError("No commit date found")
    
    except Exception as e:
        xbmc.log(f"GitLab date error: {str(e)}", xbmc.LOGERROR)
        # Only return stored value if API call completely fails
        stored_date = ADDON.getSetting('last_gitlab_date')
        return stored_date if stored_date else 'Data non disponibile'

def get_gitlab_commit_id():
    try:
        req = urllib.request.Request(
            GITLAB_API_URL,
            headers={
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
                'Referer': 'https://gitlab.com/'
            }
        )
        with urllib.request.urlopen(req, timeout=10) as response:
            data = json.loads(response.read().decode('utf-8'))
            return data.get('last_commit_id', '')[:8]  # Get first 8 chars
    except Exception as e:
        xbmc.log(f"Error getting GitLab commit ID: {str(e)}", xbmc.LOGERROR)
        return None

def update_commit_id(commit_id):
    try:
        addon_path = ADDON.getAddonInfo('path')
        addon_xml_path = os.path.join(addon_path, 'addon.xml')
        
        tree = ET.parse(addon_xml_path)
        root = tree.getroot()
        commit_elem = root.find(".//channelsjson/commit_id")
        if commit_elem is not None:
            commit_elem.text = commit_id
            tree.write(addon_xml_path, encoding='UTF-8', xml_declaration=True)
            return True
        return False
    except Exception as e:
        xbmc.log(f"Error updating commit_id: {str(e)}", xbmc.LOGERROR)
        return False

def download_file(url, dest):
    try:
        req = urllib.request.Request(
            url,
            headers={
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
                'Referer': 'https://gitlab.com/'
            }
        )
        with urllib.request.urlopen(req, timeout=10) as response:
            content = response.read()
            with open(dest, 'wb') as out_file:
                out_file.write(content)
            xbmc.log(f"File downloaded successfully to {dest}", xbmc.LOGINFO)
            return True
    except Exception as e:
        xbmc.log(f"Failed to download file: {str(e)}", xbmc.LOGERROR)
    return False

def main():
    # Check if KOD addon exists
    if not os.path.exists(KOD_ADDON_PATH):
        xbmcgui.Dialog().ok("Attenzione", "Il plugin KOD non è installato, non posso aggiornare il file!")
        return

    # Get file dates
    dateGitLab = get_gitlab_file_last_modified_date()
    dateLocal = get_file_modification_date() if os.path.exists(CHANNELS_JSON_PATH) else 'Non presente'

    # Get current GitLab commit ID
    new_commit_id = get_gitlab_commit_id()

    # Prepare dates message
    dates_message = f"Data channels.json su GitLab: {dateGitLab}\nData channels.json in locale: {dateLocal}"
    
    # Simplified status message
    status_message = "[B]Vuoi aggiornare il file channels.json?[/B]"
    
    # Copyright message in italic
    message = f"[I]Telegram: @ItalianSpaghettiGeeks[/I]\n\n{dates_message}\n\n{status_message}"
    
    # Use standard dialog
    dialog = xbmcgui.Dialog()
    if dialog.yesno('KOD domains update', message):
        if download_file(CHANNELS_JSON_URL, CHANNELS_JSON_PATH):
            # Only update the commit ID after successful download
            if new_commit_id:
                update_commit_id(new_commit_id)
            dialog.notification('Success', 'channels.json aggiornato con successo!', xbmcgui.NOTIFICATION_INFO)
        else:
            dialog.notification('Error', 'Errore durante l\'aggiornamento di channels.json', xbmcgui.NOTIFICATION_ERROR)

if __name__ == '__main__':
    main()